searchData={"items":[{"type":"module","title":"ZXCVBN","doc":"ZXCVBN is an Elixir implemenation of [ZXCVBN][zxcvbn-gh].\nThe original paper is available [here][zxcvbn-paper].\n\nThe usage is simple:\n\n    iex> ZXCVBN.zxcvbn(\"password\")\n      %{\n      calc_time: 51,\n      crack_times_display: %{\n        offline_fast_hashing_1e10_per_second: \"less than a second\",\n        offline_slow_hashing_1e4_per_second: \"less than a second\",\n        online_no_throttling_10_per_second: \"less than a second\",\n        online_throttling_100_per_hour: \"1 minute\"\n      },\n      crack_times_seconds: %{\n        offline_fast_hashing_1e10_per_second: 3.0e-10,\n        offline_slow_hashing_1e4_per_second: 0.0003,\n        online_no_throttling_10_per_second: 0.3,\n        online_throttling_100_per_hour: 108.0\n      },\n      feedback: %{\n        suggestions: [\"Add another word or two. Uncommon words are better.\",\n         \"Predictable substitutions like '@' instead of 'a' don't help very much\"],\n        warning: \"\"\n      },\n      guesses: 3,\n      guesses_log10: 0.47712125471966244,\n      password: \"password\",\n      score: 0,\n      sequence: [\n        %{\n          base_guesses: 2,\n          dictionary_name: :passwords,\n          guesses: 2,\n          guesses_log10: 0.3010299956639812,\n          i: 0,\n          j: 7,\n          l33t: false,\n          l33t_variations: 1,\n          matched_word: \"password\",\n          pattern: :dictionary,\n          rank: 2,\n          reversed: false,\n          token: \"password\",\n          uppercase_variations: 1\n        }\n      ]\n    }\n\nYou can also pass of list of user inputs\nthat will act as additional dictionary.\nThis is useful for checking if the password\nis not same or variant of associated user inputs.\n\n    iex> ZXCVBN.zxcvbn(\"nepal\", [\"samar\", \"mustang\", \"nepal\", \"hello@example.com\"])\n\n[zxcvbn-gh]: https://github.com/dropbox/zxcvbn\n[zxcvbn-paper]: https://www.usenix.org/system/files/conference/usenixsecurity16/sec16_paper_wheeler.pdf","ref":"ZXCVBN.html"},{"type":"function","title":"ZXCVBN.zxcvbn/2","doc":"","ref":"ZXCVBN.html#zxcvbn/2"},{"type":"module","title":"ZXCVBN.AdjacencyGraphs","doc":"Adjacency graph ceated by [this script][adjacency-graph-script]\n\n[adjacency-graph-script]: https://github.com/dropbox/zxcvbn/blob/cb040cd780e42adaafb998625c5a5c91db3dbaab/data-scripts/build_keyboard_adjacency_graphs.py","ref":"ZXCVBN.AdjacencyGraphs.html"},{"type":"function","title":"ZXCVBN.AdjacencyGraphs.adjacency_graph/0","doc":"","ref":"ZXCVBN.AdjacencyGraphs.html#adjacency_graph/0"},{"type":"module","title":"ZXCVBN.DefaultMessageFormatter","doc":"","ref":"ZXCVBN.DefaultMessageFormatter.html"},{"type":"function","title":"ZXCVBN.DefaultMessageFormatter.format/1","doc":"","ref":"ZXCVBN.DefaultMessageFormatter.html#format/1"},{"type":"module","title":"ZXCVBN.Feedback","doc":"Feedback module to produce warning and suggestions","ref":"ZXCVBN.Feedback.html"},{"type":"function","title":"ZXCVBN.Feedback.get_feedback/2","doc":"","ref":"ZXCVBN.Feedback.html#get_feedback/2"},{"type":"module","title":"ZXCVBN.FrequencyLists","doc":"Frequency Lists created by [this script][frequency-lists-script]\n\n[frequency-lists-script]: https://github.com/dropbox/zxcvbn/blob/cb040cd780e42adaafb998625c5a5c91db3dbaab/data-scripts/build_frequency_lists.py","ref":"ZXCVBN.FrequencyLists.html"},{"type":"function","title":"ZXCVBN.FrequencyLists.frequency_lists/0","doc":"","ref":"ZXCVBN.FrequencyLists.html#frequency_lists/0"},{"type":"module","title":"ZXCVBN.Matching","doc":"Matching module","ref":"ZXCVBN.Matching.html"},{"type":"function","title":"ZXCVBN.Matching.add_frequency_lists/2","doc":"","ref":"ZXCVBN.Matching.html#add_frequency_lists/2"},{"type":"function","title":"ZXCVBN.Matching.build_ranked_dict/1","doc":"","ref":"ZXCVBN.Matching.html#build_ranked_dict/1"},{"type":"function","title":"ZXCVBN.Matching.date_match/2","doc":"a \"date\" is recognized as:\n  any 3-tuple that starts or ends with a 2- or 4-digit year,\n  with 2 or 0 separator chars (1.1.91 or 1191),\n  maybe zero-padded (01-01-91 vs 1-1-91),\n  a month between 1 and 12,\n  a day between 1 and 31.\n\nnote: this isn't true date parsing in that \"feb 31st\" is allowed,\nthis doesn't check for leap years, etc.\n\nrecipe:\nstart with regex to find maybe-dates, then attempt to map the integers\nonto month-day-year to filter the maybe-dates into dates.\nfinally, remove matches that are substrings of other matches to reduce noise.\n\nnote: instead of using a lazy or greedy regex to find many dates over the full string,\nthis uses a ^...$ regex against every substring of the password -- less performant but leads\nto every possible date match.","ref":"ZXCVBN.Matching.html#date_match/2"},{"type":"function","title":"ZXCVBN.Matching.dedup/1","doc":"","ref":"ZXCVBN.Matching.html#dedup/1"},{"type":"function","title":"ZXCVBN.Matching.do_sequence_match/2","doc":"","ref":"ZXCVBN.Matching.html#do_sequence_match/2"},{"type":"function","title":"ZXCVBN.Matching.map_ints_to_dmy/1","doc":"given a 3-tuple, discard if:\n  - middle int is over 31 (for all dmy formats, years are never allowed in\n  the middle)\n  - middle int is zero\n  - any int is over the max allowable year\n  - any int is over two digits but under the min allowable year\n  - 2 ints are over 31, the max allowable day\n  - 2 ints are zero\n  - all ints are over 12, the max allowable month","ref":"ZXCVBN.Matching.html#map_ints_to_dmy/1"},{"type":"function","title":"ZXCVBN.Matching.omnimatch/2","doc":"omnimatch -- perform all matches","ref":"ZXCVBN.Matching.html#omnimatch/2"},{"type":"function","title":"ZXCVBN.Matching.regex_match/3","doc":"","ref":"ZXCVBN.Matching.html#regex_match/3"},{"type":"function","title":"ZXCVBN.Matching.sequence_match/2","doc":"","ref":"ZXCVBN.Matching.html#sequence_match/2"},{"type":"function","title":"ZXCVBN.Matching.spatial_match_helper/3","doc":"","ref":"ZXCVBN.Matching.html#spatial_match_helper/3"},{"type":"module","title":"ZXCVBN.MessageFormatter","doc":"","ref":"ZXCVBN.MessageFormatter.html"},{"type":"function","title":"ZXCVBN.MessageFormatter.format/1","doc":"","ref":"ZXCVBN.MessageFormatter.html#format/1"},{"type":"module","title":"ZXCVBN.Scoring","doc":"Score calculation","ref":"ZXCVBN.Scoring.html"},{"type":"function","title":"ZXCVBN.Scoring.most_guessable_match_sequence/3","doc":"------------------------------------------------------------------------------\nsearch --- most guessable match sequence -------------------------------------\n------------------------------------------------------------------------------\n\ntakes a sequence of overlapping matches, returns the non-overlapping sequence with\nminimum guesses. the following is a O(l_max * (n + m)) dynamic programming algorithm\nfor a length-n password with m candidate matches. l_max is the maximum optimal\nsequence length spanning each prefix of the password. In practice it rarely exceeds 5 and the\nsearch terminates rapidly.\n\nthe optimal \"minimum guesses\" sequence is here defined to be the sequence that\nminimizes the following function:\n\n   g = l! * Product(m.guesses for m in sequence) + D^(l - 1)\n\nwhere l is the length of the sequence.\n\nthe factorial term is the number of ways to order l patterns.\n\nthe D^(l-1) term is another length penalty, roughly capturing the idea that an\nattacker will try lower-length sequences first before trying length-l sequences.\n\nfor example, consider a sequence that is date-repeat-dictionary.\n - an attacker would need to try other date-repeat-dictionary combinations,\n   hence the product term.\n - an attacker would need to try repeat-date-dictionary, dictionary-repeat-date,\n   ..., hence the factorial term.\n - an attacker would also likely try length-1 (dictionary) and length-2 (dictionary-date)\n   sequences before length-3. assuming at minimum D guesses per pattern type,\n   D^(l-1) approximates Sum(D^i for i in [1..l-1]\n\n------------------------------------------------------------------------------","ref":"ZXCVBN.Scoring.html#most_guessable_match_sequence/3"},{"type":"function","title":"ZXCVBN.Scoring.nCk/2","doc":"Reference: http://blog.plover.com/math/choose.html","ref":"ZXCVBN.Scoring.html#nCk/2"},{"type":"function","title":"ZXCVBN.Scoring.re_all_upper/0","doc":"","ref":"ZXCVBN.Scoring.html#re_all_upper/0"},{"type":"function","title":"ZXCVBN.Scoring.re_start_upper/0","doc":"","ref":"ZXCVBN.Scoring.html#re_start_upper/0"},{"type":"function","title":"ZXCVBN.Scoring.reference_year/0","doc":"","ref":"ZXCVBN.Scoring.html#reference_year/0"},{"type":"function","title":"ZXCVBN.Scoring.spatial_guesses/1","doc":"","ref":"ZXCVBN.Scoring.html#spatial_guesses/1"},{"type":"module","title":"ZXCVBN.TimeEstimates","doc":"Calculate various attacking times","ref":"ZXCVBN.TimeEstimates.html"},{"type":"function","title":"ZXCVBN.TimeEstimates.estimate_attack_times/1","doc":"","ref":"ZXCVBN.TimeEstimates.html#estimate_attack_times/1"},{"type":"module","title":"ZXCVBN.Utils","doc":"Helper functions","ref":"ZXCVBN.Utils.html"},{"type":"function","title":"ZXCVBN.Utils.calc_average_degree/1","doc":"","ref":"ZXCVBN.Utils.html#calc_average_degree/1"},{"type":"function","title":"ZXCVBN.Utils.downcase/1","doc":"","ref":"ZXCVBN.Utils.html#downcase/1"},{"type":"function","title":"ZXCVBN.Utils.factorial/1","doc":"","ref":"ZXCVBN.Utils.html#factorial/1"},{"type":"function","title":"ZXCVBN.Utils.file_to_list/1","doc":"","ref":"ZXCVBN.Utils.html#file_to_list/1"},{"type":"function","title":"ZXCVBN.Utils.pow/2","doc":"","ref":"ZXCVBN.Utils.html#pow/2"},{"type":"function","title":"ZXCVBN.Utils.slice/2","doc":"","ref":"ZXCVBN.Utils.html#slice/2"},{"type":"function","title":"ZXCVBN.Utils.slice/3","doc":"","ref":"ZXCVBN.Utils.html#slice/3"},{"type":"function","title":"ZXCVBN.Utils.strlen/1","doc":"","ref":"ZXCVBN.Utils.html#strlen/1"},{"type":"function","title":"ZXCVBN.Utils.strlen_bruteforce/1","doc":"","ref":"ZXCVBN.Utils.html#strlen_bruteforce/1"},{"type":"function","title":"ZXCVBN.Utils.time/0","doc":"","ref":"ZXCVBN.Utils.html#time/0"},{"type":"extras","title":"zxcvbn-elixir","doc":"# zxcvbn-elixir [![Build Status](https://travis-ci.org/techgaun/zxcvbn-elixir.svg?branch=master)](https://travis-ci.org/techgaun/zxcvbn-elixir) [![Hex Version](https://img.shields.io/hexpm/v/zxcvbn.svg)](https://hex.pm/packages/zxcvbn)\n\n> Elixir implementation of [zxcvbn](https://github.com/dropbox/zxcvbn) by dropbox","ref":"readme.html"},{"type":"extras","title":"Installation - zxcvbn-elixir","doc":"```elixir\ndef deps do\n  [\n    {:zxcvbn, \"~> 0.3.0\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Usage - zxcvbn-elixir","doc":"```elixir\nimport ZXCVBN\n\n# default without user inputs\n\nzxcvbn(\"Some Password\")\n\n# with user inputs; useful for adding to dictionary (for eg. submitted form inputs;\n# think someone using their e-mail address as password for example)\nzxcvbn(\"Password1\", [\"user@email.com\", \"Nepal\", \"Kathmandu\"])\n```","ref":"readme.html#usage"},{"type":"extras","title":"Configuration - zxcvbn-elixir","doc":"zxcvbn can be limited to only ascii charsets by setting mode in config as below:\n\n```elixir\nconfig :zxcvbn,\n  mode: :ascii\n```\n\nzxcvbn allow to customize feedback messages, for example to add I18n feature.\n\n```elixir\nconfig :zxcvbn,\n  message_formatter: MyApp.ZXCVBNMessageFormatter\n```\n\n```elixir\ndefmodule MyApp.ZXCVBNMessageFormatter do\n  def format(str) do\n    Gettext.dgettext(MyAppWeb.Gettext, \"zxcvbn\", str)\n  end\nend\n```","ref":"readme.html#configuration"},{"type":"extras","title":"Usage Notes - zxcvbn-elixir","doc":"- Ideally, when you are using ZXCVBN, pass the first 100-200 characters only for reasonable latency.","ref":"readme.html#usage-notes"},{"type":"extras","title":"Output Description - zxcvbn-elixir","doc":"The result of running `ZXCVBN.zxcvbn/1` and `ZXCVBN.zxcvbn/2` is a map\nexcept for when empty string is supplied as password (which returns `:error`).\n\nBelow is a sample result and description of each fields.\n\n```elixir\n%{\n  # how long it took zxcvbn to calculate an answer,\n  # in milliseconds.\n  calc_time: 51,\n\n  # same keys as result.crack_times_seconds,\n  # with friendlier display string values:\n  # \"less than a second\", \"3 hours\", \"centuries\", etc.\n\n  crack_times_display: %{\n    offline_fast_hashing_1e10_per_second: \"less than a second\",\n    offline_slow_hashing_1e4_per_second: \"less than a second\",\n    online_no_throttling_10_per_second: \"less than a second\",\n    online_throttling_100_per_hour: \"1 minute\"\n  },\n\n  # dictionary of back-of-the-envelope crack time\n  # estimations, in seconds, based on a few scenarios\n\n  crack_times_seconds: %{\n    # offline attack with user-unique salting but a fast hash\n    # function like SHA-1, SHA-256 or MD5. A wide range of\n    # reasonable numbers anywhere from one billion - one trillion\n    # guesses per second, depending on number of cores and machines.\n    # ballparking at 10B/sec.\n\n    offline_fast_hashing_1e10_per_second: 3.0e-10,\n\n    # offline attack. assumes multiple attackers,\n    # proper user-unique salting, and a slow hash function\n    # w/ moderate work factor, such as bcrypt, scrypt, PBKDF2.\n\n    offline_slow_hashing_1e4_per_second: 0.0003,\n\n    # online attack on a service that doesn't ratelimit,\n    # or where an attacker has outsmarted ratelimiting.\n\n    online_no_throttling_10_per_second: 0.3,\n\n    # online attack on a service that ratelimits password auth attempts\n\n    online_throttling_100_per_hour: 108.0\n  },\n\n  # verbal feedback to help choose better passwords. set when score <= 2.\n  feedback: %{\n\n    # a possibly-empty list of suggestions to help choose a less\n    # guessable password. eg. 'Add another word or two'\n    suggestions: [\"Add another word or two. Uncommon words are better.\",\n     \"Predictable substitutions like '@' instead of 'a' don't help very much\"],\n\n    # explains what's wrong, eg. 'this is a top-10 common password'.\n    # not always set -- sometimes an empty string\n    warning: \"\"\n  },\n\n  # estimated guesses needed to crack password\n  guesses: 3,\n\n  # order of magnitude of guesses\n  guesses_log10: 0.47712125471966244,\n\n  # input password\n  password: \"password\",\n\n  # Integer from 0-4 (useful for implementing a strength bar)\n  # 0 too guessable: risky password. (guesses < 10^3)\n  # 1 very guessable: protection from throttled online attacks. (guesses < 10^6)\n  # 2 somewhat guessable: protection from unthrottled online attacks. (guesses < 10^8)\n  # 3 safely unguessable: moderate protection from offline slow-hash scenario. (guesses < 10^10)\n  # 4 very unguessable: strong protection from offline slow-hash scenario. (guesses >= 10^10)\n  score: 0,\n\n  # the list of patterns that zxcvbn based the\n  # guess calculation on.\n  sequence: [\n    %{\n      base_guesses: 2,\n      dictionary_name: :passwords,\n      guesses: 2,\n      guesses_log10: 0.3010299956639812,\n      i: 0,\n      j: 7,\n      l33t: false,\n      l33t_variations: 1,\n      matched_word: \"password\",\n      pattern: :dictionary,\n      rank: 2,\n      reversed: false,\n      token: \"password\",\n      uppercase_variations: 1\n    }\n  ]\n}\n```","ref":"readme.html#output-description"},{"type":"extras","title":"Benchmark - zxcvbn-elixir","doc":"The basic benchmark result can be seen by running:\n\n```shell\nmix run bench/run.exs\n```\n\nThat will create html output on `benchmarks` directory.","ref":"readme.html#benchmark"},{"type":"extras","title":"Test - zxcvbn-elixir","doc":"The [zxcvbn.js](zxcvbn.js) is used to test the [correctness](test/zxcvbn_test.exs)\nwith the original implementation. ZXCVBN is a pure Elixir implementation\nand not a wrapper on top of javascript version.","ref":"readme.html#test"},{"type":"extras","title":"Test Setup - zxcvbn-elixir","doc":"```\nnpm install\nmix test\n```","ref":"readme.html#test-setup"},{"type":"extras","title":"Author - zxcvbn-elixir","doc":"- [Samar Acharya](https://github.com/techgaun)","ref":"readme.html#author"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}